<?php

declare(strict_types = 1);

namespace Drupal\migrate_dc\Plugin\migrate\source;

use Drupal\Component\Serialization\SerializationInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\migrate\Plugin\migrate\source\SourcePluginBase;
use Drupal\migrate\Plugin\MigrationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Reads source from a JSON file.
 *
 * @\Drupal\migrate\Annotation\MigrateSource(
 *   id = "migrate_dc_json"
 * )
 */
class MigrateDcJsonSource extends SourcePluginBase implements ContainerFactoryPluginInterface {

  /**
   * Path to the directory which contains the source file.
   */
  protected string $fileDir = '';

  /**
   * File name relative from $this->fileDir.
   */
  protected string $fileName = '';

  /**
   * An array of source fields.
   */
  protected array $fields = [];

  /**
   * The field name that is a unique identifier.
   */
  protected string|array $identifier = '';

  /**
   * Items to import.
   */
  protected ?array $items = NULL;

  protected SerializationInterface $serializer;

  /**
   * @var array<string>
   */
  protected array $requiredConfigKeys = [
    'fileDir',
    'fileName',
    'fields',
    'identifier',
  ];

  /**
   * @var array<string, string>
   */
  protected array $configKeyMapping = [
    'fileDir' => 'fileDir',
    'fileName' => 'fileName',
    'fields' => 'fields',
    'identifier' => 'identifier',
  ];

  /**
   * {@inheritDoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
    MigrationInterface $migration = NULL,
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $migration,
      $container->get('serialization.json'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    MigrationInterface $migration,
    SerializationInterface $serializer,
  ) {
    $this->serializer = $serializer;

    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration);

    $this
      ->assertConfiguration()
      ->setPropertiesFromConfiguration();
  }

  protected function assertConfiguration(): static {
    $missingKeys = array_diff(
      $this->requiredConfigKeys,
      array_keys($this->configuration)
    );

    assert(
      count($missingKeys) === 0,
      'Source configuration must include the following keys: ' . implode(', ', $missingKeys)
    );

    return $this;
  }

  protected function setPropertiesFromConfiguration(): static {
    foreach ($this->configKeyMapping as $property => $config) {
      if (array_key_exists($config, $this->configuration)) {
        $this->$property = $this->configuration[$config];
      }
    }

    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function count($refresh = FALSE) {
    return count($this->getItems());
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    if (is_array($this->identifier)) {
      return $this->identifier;
    }

    return [
      $this->identifier => [
        'type' => 'string',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function __toString() {
    return $this->getFilePath();
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    return $this->fields;
  }

  /**
   * Get protected values.
   *
   * @todo Do we need this?
   *
   * @param string $property
   *   Property name.
   *
   * @return mixed
   *   Value of the property.
   */
  public function get(string $property) {
    return $this->{$property};
  }

  /**
   * Get all items.
   */
  protected function getItems(): array {
    if ($this->items === NULL) {
      $this->initItems();
    }

    return $this->items;
  }

  protected function initItems(): static {
    $this->items = $this->serializer->decode($this->fileGetContents($this->getFilePath()));
    $ids = $this->getIds();
    if (count($ids) === 1 && key($this->items) !== 0) {
      $idField = key($ids);
      foreach (array_keys($this->items) as $id) {
        $this->items[$id][$idField] = $id;
      }
    }

    return $this;
  }

  /**
   * {@inheritdoc}
   */
  protected function initializeIterator() {
    return new \ArrayIterator($this->getItems());
  }

  protected function getFilePath(): string {
    return "{$this->fileDir}/{$this->fileName}";
  }

  protected function fileGetContents(string $fileName): string {
    $content = file_get_contents($fileName);
    if ($content === FALSE) {
      throw new \RuntimeException("file '$fileName' could not be read", 1);
    }

    return $content;
  }

}
