<?php

declare(strict_types = 1);

namespace Drupal\Tests\migrate_dc\Kernel\Plugin\migrate\process;

use Drupal;
use Drupal\file\FileInterface;
use Drupal\migrate\MigrateException;
use Drupal\migrate\MigrateExecutable;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\MigrateMessageInterface;
use Drupal\migrate\Plugin\MigrationPluginManagerInterface;
use Drupal\migrate\Row as MigrateRow;
use Drupal\migrate_dc\Plugin\migrate\process\MigrateDcFileContent;
use Drupal\KernelTests\KernelTestBase;

/**
 * @group migrate_dc
 *
 * @covers \Drupal\migrate_dc\Plugin\migrate\process\MigrateDcFileContent
 */
class MigrateDcFileContentTest extends KernelTestBase implements MigrateMessageInterface {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'migrate_dc',
    'migrate',
    'file',
    'system',
    'user',
    'profile',
    'views',
  ];

  protected array $configuration = [];

  protected string $pluginId = '';

  protected array $pluginDefinition = [];

  protected MigrateRow $row;

  protected MigrateExecutableInterface $migrateExecutable;

  protected mixed $destinationProperty = NULL;

  protected MigrationPluginManagerInterface $migrationPluginManager;

  protected FileInterface $file;

  protected string $fileContent = 'dummy text';

  protected string $filePath = 'public://';

  protected string $fileName = 'fileName.txt';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('file');
    $this->installEntitySchema('user');
    $this->installConfig(static::$modules);

    $this->row = new MigrateRow();
    $this->destinationProperty = NULL;

    $this->migrationPluginManager = Drupal::service('plugin.manager.migration');
    /** @var \Drupal\migrate\Plugin\Migration $migration */
    $migration = $this->migrationPluginManager->createStubMigration([]);
    $this->migrateExecutable = new MigrateExecutable(
      $migration,
      // @todo Create a Dummy messenger.
      $this,
    );

    file_put_contents($this->filePath . $this->fileName, $this->fileContent);
  }

  public function casesTransformSuccess(): array {
    return [
      'file exists' => [
        $this->fileContent,
        [
          'path' => $this->filePath,
          'file_name' => $this->fileName,
        ],
        [
          'property' => '',
          'value' => '',
        ],
      ],
      'file exists with @ in path' => [
        $this->fileContent,
        [
          'path' => '@a/b/c',
          'file_name' => $this->fileName,
        ],
        [
          'property' => 'a/b/c',
          'value' => $this->filePath,
        ],
      ],
      'file does not exist' => [
        '',
        [
          'path' => 'foo',
          'file_name' => "bar",
        ],
        [
          'property' => '',
          'value' => '',
        ],
      ],
    ];
  }

  /**
   * @dataProvider casesTransformSuccess
   */
  public function testTransformSuccess($expected, $configuration, $destination): void {
    $value = '';
    $this->row->setDestinationProperty($destination['property'], $destination['value']);

    $processor = new MigrateDcFileContent(
      $configuration,
      $this->pluginId,
      $this->pluginDefinition
    );

    static::assertSame(
      $expected,
      $processor
        ->transform(
          $value,
          $this->migrateExecutable,
          $this->row,
          $this->destinationProperty
      )
    );
  }

  public function casesTransformFail(): array {
    return [
      'no file in config' => [
        MigrateException::class,
        [],
        [
          'property' => '',
          'value' => '',
        ],
      ],
    ];
  }

  /**
   * @dataProvider casesTransformFail
   */
  public function testTransformFail($expected, $configuration, $destination): void {
    $value = '';
    $this->row->setDestinationProperty($destination['property'], $destination['value']);

    $processor = new MigrateDcFileContent(
      $configuration,
      $this->pluginId,
      $this->pluginDefinition
    );

    $this->expectException($expected);
    $this->expectExceptionMessage('Config path value does not exist');

    $processor->transform(
      $value,
      $this->migrateExecutable,
      $this->row,
      $this->destinationProperty
    );
  }

  /**
   * {@inheritdoc}
   *
   * @todo Move this into a separated helper class.
   */
  public function display($message, $type = 'status') {
    static::assertTrue($type === 'status', $message);
  }

}
